"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const ymir_1 = require("@fimbul/ymir");
const tsutils_1 = require("tsutils");
const utils_1 = require("../utils");
const ts = require("typescript");
let Rule = class Rule extends ymir_1.TypedRule {
    apply() {
        const re = /\basync\b/g;
        let wrappedAst;
        for (let match = re.exec(this.sourceFile.text); match !== null; match = re.exec(this.sourceFile.text)) {
            const { node } = tsutils_1.getWrappedNodeAtPosition(wrappedAst || (wrappedAst = this.context.getWrappedAst()), match.index);
            if (node.kind !== ts.SyntaxKind.AsyncKeyword || node.end !== re.lastIndex)
                continue;
            const parent = node.parent;
            if (utils_1.isAsyncFunction(parent))
                parent.body.statements.forEach(this.visitStatement, this);
        }
    }
    visitStatement(node) {
        if (tsutils_1.isExpressionStatement(node)) {
            if (tsutils_1.isCallExpression(node.expression) && tsutils_1.isThenableType(this.checker, node.expression))
                this.addFindingAtNode(node, "Return value of async function call was discarded. Did you mean to 'await' its result?");
            return;
        }
        for (const statement of utils_1.childStatements(node))
            this.visitStatement(statement);
    }
};
Rule = tslib_1.__decorate([
    ymir_1.excludeDeclarationFiles
], Rule);
exports.Rule = Rule;
//# sourceMappingURL=await-async-result.js.map