"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const ymir_1 = require("@fimbul/ymir");
const ts = require("typescript");
const tsutils_1 = require("tsutils");
const path = require("path");
let Rule = class Rule extends ymir_1.TypedRule {
    constructor() {
        super(...arguments);
        this.window = undefined;
    }
    apply() {
        for (const node of this.context.getFlatAst()) {
            if (node.kind !== ts.SyntaxKind.Identifier)
                continue;
            const text = node.text;
            if (isWhitelisted(text) || (tsutils_1.getUsageDomain(node) & tsutils_1.UsageDomain.Value) === 0)
                continue;
            const symbol = this.checker.getSymbolAtLocation(node);
            if (symbol !== undefined &&
                symbol.valueDeclaration !== undefined &&
                symbol.flags & (ts.SymbolFlags.Variable | ts.SymbolFlags.Function) &&
                this.isDomGlobal(symbol.valueDeclaration, text))
                this.addFindingAtNode(node, `Referencing global '${text}' is not allowed. Did you mean to use a local variable or parameter with a similar name?`);
        }
    }
    isDomGlobal(declaration, name) {
        const sourceFile = getLibFile(declaration);
        return sourceFile !== undefined && this.isWindowProperty(sourceFile, name);
    }
    isWindowProperty(sourceFile, name) {
        if (this.window === undefined) {
            const declaration = sourceFile.statements.find((d) => tsutils_1.isInterfaceDeclaration(d) && d.name.text === 'Window');
            this.window = this.checker.getDeclaredTypeOfSymbol(this.checker.getSymbolAtLocation(declaration.name));
        }
        return this.window.getProperty(name) !== undefined;
    }
};
Rule = tslib_1.__decorate([
    ymir_1.excludeDeclarationFiles
], Rule);
exports.Rule = Rule;
function getLibFile(node) {
    if (node.kind === ts.SyntaxKind.FunctionDeclaration) {
        node = node.parent;
    }
    else if (node.kind === ts.SyntaxKind.VariableDeclaration) {
        node = node.parent.parent;
        if (node.kind !== ts.SyntaxKind.VariableStatement)
            return;
        node = node.parent;
    }
    else {
        return;
    }
    return tsutils_1.isSourceFile(node) && /^lib(?:\.dom)?\.d\.ts$/.test(path.basename(node.fileName)) ? node : undefined;
}
function isWhitelisted(name) {
    // exclude all identifiers that start with an uppercase letter to allow `new Event()` and stuff
    if (name.charAt(0) === name.charAt(0).toUpperCase())
        return true;
    switch (name) {
        case 'document':
        case 'window':
        case 'navigator':
        case 'alert':
        case 'confirm':
        case 'prompt':
        case 'atob':
        case 'btoa':
        case 'fetch':
        case 'console':
        case 'sessionStorage':
        case 'localStorage':
        case 'indexedDB':
            return true;
        default:
            return /^(?:(?:set|clear)(?:Timeout|Interval|Immediate)|(?:request|cancel)AnimationFrame)$/.test(name);
    }
}
//# sourceMappingURL=ban-dom-globals.js.map