"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const ymir_1 = require("@fimbul/ymir");
const tsutils_1 = require("tsutils");
const ts = require("typescript");
const utils_1 = require("../utils");
let Rule = class Rule extends ymir_1.TypedRule {
    apply() {
        const re = /\bdelete\b/g;
        let wrappedAst;
        for (let match = re.exec(this.sourceFile.text); match !== null; match = re.exec(this.sourceFile.text)) {
            const { node } = tsutils_1.getWrappedNodeAtPosition(wrappedAst || (wrappedAst = this.context.getWrappedAst()), match.index);
            if (!tsutils_1.isDeleteExpression(node) || node.expression.pos !== re.lastIndex)
                continue;
            const { expression } = node;
            if (tsutils_1.isElementAccessExpression(expression)) {
                for (const { symbol, name } of utils_1.elementAccessSymbols(expression, this.checker))
                    this.checkSymbol(symbol, node, name);
            }
            else {
                this.checkSymbol(this.checker.getSymbolAtLocation(expression), node);
            }
        }
    }
    checkSymbol(symbol, errorNode, name) {
        if (symbol === undefined || symbol.flags & ts.SymbolFlags.Optional)
            return;
        this.addFindingAtNode(errorNode, `Only 'delete' optional properties. Property '${name || symbol.name}' is required.`);
    }
};
Rule = tslib_1.__decorate([
    ymir_1.excludeDeclarationFiles,
    ymir_1.requiresCompilerOption('strictNullChecks')
], Rule);
exports.Rule = Rule;
//# sourceMappingURL=delete-only-optional-property.js.map