"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const ts = require("typescript");
const tsutils_1 = require("tsutils");
const ymir_1 = require("@fimbul/ymir");
let Rule = class Rule extends ymir_1.AbstractRule {
    constructor() {
        super(...arguments);
        this.containsYield = false;
    }
    apply() {
        return this.iterate(this.context.getWrappedAst().next, undefined);
    }
    iterate(wrap, end) {
        do { // iterate as linked list until we find a generator
            if (wrap.kind === ts.SyntaxKind.Block && isGenerator(wrap.node.parent)) {
                this.containsYield = false;
                wrap.children.forEach(this.visitNode, this); // walk the function body recursively
                if (this.shouldFail()) // call as method so CFA doesn't infer `this.containsYield` as always false
                    this.fail(wrap.node.parent);
                wrap = wrap.skip; // continue right after the function body
            }
            else {
                wrap = wrap.next;
            }
        } while (wrap !== end);
    }
    visitNode(wrap) {
        if (wrap.node.kind === ts.SyntaxKind.YieldExpression) {
            this.containsYield = true;
            if (wrap.children.length === 0)
                return;
            return this.visitNode(wrap.children[0]);
        }
        if (tsutils_1.isFunctionScopeBoundary(wrap.node)) {
            const saved = this.containsYield;
            // can iterate as linked list again for nested functions
            this.iterate(wrap.next, wrap.skip);
            this.containsYield = saved;
            return;
        }
        return wrap.children.forEach(this.visitNode, this);
    }
    shouldFail() {
        return !this.containsYield;
    }
    fail({ asteriskToken, name }) {
        this.addFinding(asteriskToken.end - 1, asteriskToken.end, `Generator ${name === undefined ? '' : `'${name.getText(this.sourceFile).replace(/'/g, "\\'")}' `}contains no 'yield'.`);
    }
};
Rule = tslib_1.__decorate([
    ymir_1.excludeDeclarationFiles
], Rule);
exports.Rule = Rule;
function isGenerator(node) {
    switch (node.kind) {
        case ts.SyntaxKind.FunctionDeclaration:
        case ts.SyntaxKind.FunctionExpression:
        case ts.SyntaxKind.MethodDeclaration:
            return node.asteriskToken !== undefined;
        default:
            return false;
    }
}
//# sourceMappingURL=generator-require-yield.js.map