"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const ymir_1 = require("@fimbul/ymir");
const ts = require("typescript");
const tsutils_1 = require("tsutils");
const utils_1 = require("../utils");
let Rule = class Rule extends ymir_1.AbstractRule {
    apply() {
        for (const { caseBlock: { clauses } } of utils_1.switchStatements(this.context))
            for (const clause of clauses)
                for (const statement of clause.statements)
                    if (isForbiddenDeclaration(statement))
                        this.addFindingAtNode(statement, 'Unexpected lexical declaration in case block.');
    }
};
Rule = tslib_1.__decorate([
    ymir_1.excludeDeclarationFiles
], Rule);
exports.Rule = Rule;
function isForbiddenDeclaration(node) {
    switch (node.kind) {
        case ts.SyntaxKind.ClassDeclaration:
            return true;
        case ts.SyntaxKind.VariableStatement:
            return (node.declarationList.flags & ts.NodeFlags.Let) !== 0;
        case ts.SyntaxKind.EnumDeclaration:
            return !tsutils_1.hasModifier(node.modifiers, ts.SyntaxKind.ConstKeyword);
        default:
            return false;
    }
}
//# sourceMappingURL=no-case-declaration.js.map