"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const ymir_1 = require("@fimbul/ymir");
const ts = require("typescript");
const tsutils_1 = require("tsutils");
let Rule = class Rule extends ymir_1.AbstractRule {
    apply() {
        const re = /\bdebugger\s*(?:[/;]|$)/mg;
        let wrappedAst;
        const text = this.sourceFile.text;
        for (let match = re.exec(text); match !== null; match = re.exec(text)) {
            const { node } = tsutils_1.getWrappedNodeAtPosition(wrappedAst || (wrappedAst = this.context.getWrappedAst()), match.index);
            if (node.kind === ts.SyntaxKind.DebuggerStatement) {
                const start = node.getStart(this.sourceFile);
                if (start === match.index)
                    this.addFinding(start, node.end, "'debugger' statements are forbidden.", canSafelyRemoveStatement(node)
                        ? ymir_1.Replacement.delete(node.pos, node.end)
                        : { start, end: node.end, text: ';' });
            }
        }
    }
};
Rule = tslib_1.__decorate([
    ymir_1.excludeDeclarationFiles
], Rule);
exports.Rule = Rule;
function canSafelyRemoveStatement(statement) {
    switch (statement.parent.kind) {
        case ts.SyntaxKind.IfStatement:
        case ts.SyntaxKind.WithStatement:
        case ts.SyntaxKind.LabeledStatement:
            return false;
        default:
            return !tsutils_1.isIterationStatement(statement.parent);
    }
}
//# sourceMappingURL=no-debugger.js.map