"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const ymir_1 = require("@fimbul/ymir");
const ts = require("typescript");
const tsutils_1 = require("tsutils");
const utils_1 = require("../utils");
let Rule = class Rule extends ymir_1.AbstractRule {
    apply() {
        for (const { caseBlock: { clauses } } of utils_1.switchStatements(this.context)) {
            for (let i = 1; i < clauses.length; ++i) {
                if (clauses[i - 1].statements.length !== 0 &&
                    !ts.forEachLeadingCommentRange(this.sourceFile.text, clauses[i].pos, isFallthroughComment, this.sourceFile.text) &&
                    !tsutils_1.endsControlFlow(clauses[i - 1])) {
                    const kind = clauses[i].kind === ts.SyntaxKind.CaseClause ? 'case' : 'default';
                    const start = clauses[i].getStart(this.sourceFile);
                    this.addFinding(start, start + kind.length, `Missing 'break' before '${kind}'.`);
                }
            }
        }
    }
};
Rule = tslib_1.__decorate([
    ymir_1.excludeDeclarationFiles
], Rule);
exports.Rule = Rule;
function isFallthroughComment(pos, end, _kind, _newline, text) {
    return /^\s*falls? ?through\b/i.test(text.substring(pos + 2, end));
}
//# sourceMappingURL=no-fallthrough.js.map