"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const ymir_1 = require("@fimbul/ymir");
const tsutils_1 = require("tsutils");
let Rule = class Rule extends ymir_1.AbstractRule {
    apply() {
        for (const node of this.context.getFlatAst())
            if (tsutils_1.isSignatureDeclaration(node) && node.typeParameters !== undefined)
                this.checkTypeParameters(node.typeParameters, node);
    }
    checkTypeParameters(typeParameters, signature) {
        if (this.usage === undefined)
            this.usage = tsutils_1.collectVariableUsage(this.sourceFile);
        outer: for (const typeParameter of typeParameters) {
            let usedInParameters = false;
            let usedInReturnOrExtends = tsutils_1.isFunctionWithBody(signature);
            for (const use of this.usage.get(typeParameter.name).uses) {
                if (use.location.pos > signature.parameters.pos && use.location.pos < signature.parameters.end) {
                    if (usedInParameters)
                        continue outer;
                    usedInParameters = true;
                }
                else if (!usedInReturnOrExtends) {
                    usedInReturnOrExtends = use.location.pos > signature.parameters.end || isUsedInConstraint(use.location, typeParameters);
                }
            }
            if (!usedInParameters) {
                this.addFindingAtNode(typeParameter, `TypeParameter '${typeParameter.name.text}' cannot be inferred from any parameter.`);
            }
            else if (!usedInReturnOrExtends && !this.isConstrainedByOtherTypeParameter(typeParameter, typeParameters)) {
                this.addFindingAtNode(typeParameter, `TypeParameter '${typeParameter.name.text}' is not used to enforce a constraint between types and can be replaced with \
'${typeParameter.constraint ? typeParameter.constraint.getText(this.sourceFile) : 'any'}'.`);
            }
        }
    }
    isConstrainedByOtherTypeParameter(current, all) {
        if (current.constraint === undefined)
            return false;
        for (const typeParameter of all) {
            if (typeParameter === current)
                continue;
            for (const use of this.usage.get(typeParameter.name).uses)
                if (use.location.pos >= current.constraint.pos && use.location.pos < current.constraint.end)
                    return true;
        }
        return false;
    }
};
Rule = tslib_1.__decorate([
    ymir_1.typescriptOnly
], Rule);
exports.Rule = Rule;
function isUsedInConstraint(use, typeParameters) {
    for (const typeParameter of typeParameters)
        if (typeParameter.constraint !== undefined && use.pos >= typeParameter.constraint.pos && use.pos < typeParameter.constraint.end)
            return true;
    return false;
}
//# sourceMappingURL=no-misused-generics.js.map