"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const ymir_1 = require("@fimbul/ymir");
const ts = require("typescript");
const tsutils_1 = require("tsutils");
let Rule = class Rule extends ymir_1.AbstractRule {
    apply() {
        const re = /\bNaN\b/g;
        let wrappedAst;
        for (let match = re.exec(this.sourceFile.text); match !== null; match = re.exec(this.sourceFile.text)) {
            const { node } = tsutils_1.getWrappedNodeAtPosition(wrappedAst || (wrappedAst = this.context.getWrappedAst()), match.index);
            if (!tsutils_1.isIdentifier(node) || node.text !== 'NaN' || node.end !== match.index + 3)
                continue;
            let parent = node.parent;
            if (tsutils_1.isPropertyAccessExpression(parent)) {
                if (!tsutils_1.isIdentifier(parent.expression) || parent.expression.text !== 'Number')
                    continue;
                parent = parent.parent;
            }
            if (parent.kind === ts.SyntaxKind.CaseClause || tsutils_1.isBinaryExpression(parent) && isEqualityCheck(parent.operatorToken.kind))
                this.addFindingAtNode(parent, "Comparing with 'NaN' always yields 'false'. Consider using 'isNaN' instead.");
        }
    }
};
Rule = tslib_1.__decorate([
    ymir_1.excludeDeclarationFiles
], Rule);
exports.Rule = Rule;
function isEqualityCheck(kind) {
    switch (kind) {
        case ts.SyntaxKind.ExclamationEqualsEqualsToken:
        case ts.SyntaxKind.ExclamationEqualsToken:
        case ts.SyntaxKind.EqualsEqualsEqualsToken:
        case ts.SyntaxKind.EqualsEqualsToken:
            return true;
        default:
            return false;
    }
}
//# sourceMappingURL=no-nan-compare.js.map