"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const ymir_1 = require("@fimbul/ymir");
const tsutils_1 = require("tsutils");
const ts = require("typescript");
let Rule = class Rule extends ymir_1.AbstractRule {
    apply() {
        const re = /(\\*)\\(?:0[0-7]{1,2}|[1-3][0-7]{0,2}|[4-7][0-7]?)/g;
        let wrappedAst;
        for (let match = re.exec(this.sourceFile.text); match !== null; match = re.exec(this.sourceFile.text)) {
            if (match[1].length & 1) // only check if backslash is not escaped
                continue;
            const { node } = tsutils_1.getWrappedNodeAtPosition(wrappedAst || (wrappedAst = this.context.getWrappedAst()), match.index);
            switch (node.kind) {
                case ts.SyntaxKind.StringLiteral:
                case ts.SyntaxKind.NoSubstitutionTemplateLiteral:
                case ts.SyntaxKind.TemplateHead:
                case ts.SyntaxKind.TemplateMiddle:
                case ts.SyntaxKind.TemplateTail:
                    if (match.index >= node.getStart(this.sourceFile))
                        this.addFinding(match.index + match[1].length, re.lastIndex, 'Octal escape sequences are deprecated and not allowed in strict mode.', ymir_1.Replacement.replace(match.index + match[1].length + 1, re.lastIndex, `x${toHexSequence(parseInt(match[0].substr(match[1].length + 1), 8))}`));
            }
        }
    }
};
Rule = tslib_1.__decorate([
    ymir_1.excludeDeclarationFiles
], Rule);
exports.Rule = Rule;
function toHexSequence(num) {
    const result = num.toString(16);
    return (result.length === 2 ? '' : '0') + result;
}
//# sourceMappingURL=no-octal-escape.js.map