"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const ymir_1 = require("@fimbul/ymir");
const ts = require("typescript");
const tsutils_1 = require("tsutils");
const utils_1 = require("../utils");
const FAIL_MESSAGE = 'Awaiting the returned value is redundant as it is wrapped in a Promise anyway.';
let Rule = class Rule extends ymir_1.AbstractRule {
    apply() {
        const re = /(?:^|\|\||&&|return|=>|\*\/|[,(?:])\s*await\b/mg;
        let wrappedAst;
        for (let match = re.exec(this.sourceFile.text); match !== null; match = re.exec(this.sourceFile.text)) {
            const { node } = tsutils_1.getWrappedNodeAtPosition(wrappedAst || (wrappedAst = this.context.getWrappedAst()), re.lastIndex - 1);
            if (tsutils_1.isAwaitExpression(node) && re.lastIndex === node.expression.pos && isUnnecessaryAwait(node)) {
                const keywordStart = node.expression.pos - 'await'.length;
                const replacements = [ymir_1.Replacement.delete(keywordStart, node.expression.getStart(this.sourceFile))];
                if (utils_1.expressionNeedsParensWhenReplacingNode(node.expression, node))
                    replacements.push(ymir_1.Replacement.append(keywordStart, '('), ymir_1.Replacement.append(node.expression.end, ')'));
                this.addFinding(keywordStart, node.expression.pos, FAIL_MESSAGE, replacements);
            }
        }
    }
};
Rule = tslib_1.__decorate([
    ymir_1.excludeDeclarationFiles
], Rule);
exports.Rule = Rule;
function isUnnecessaryAwait(node) {
    while (true) {
        const parent = node.parent;
        outer: switch (parent.kind) {
            case ts.SyntaxKind.ArrowFunction:
                return true;
            case ts.SyntaxKind.ReturnStatement:
                return !isInsideTryBlock(parent.parent);
            case ts.SyntaxKind.ParenthesizedExpression:
                break;
            case ts.SyntaxKind.ConditionalExpression:
                if (parent.condition === node)
                    return false;
                break;
            case ts.SyntaxKind.BinaryExpression:
                if (parent.right === node) {
                    switch (parent.operatorToken.kind) {
                        case ts.SyntaxKind.AmpersandAmpersandToken:
                        case ts.SyntaxKind.BarBarToken:
                        case ts.SyntaxKind.CommaToken:
                            break outer;
                    }
                }
                return false;
            default:
                return false;
        }
        node = parent;
    }
}
function isInsideTryBlock(node) {
    while (node.parent !== undefined) {
        if (tsutils_1.isFunctionScopeBoundary(node))
            return false; // stop at function boundaries
        if (tsutils_1.isTryStatement(node.parent)) {
            if (
            // statements inside the try block always have an error handler, either catch or finally
            node.parent.tryBlock === node ||
                // return await inside the catch block is allowed if there is a finally block
                // otherwise the finally block is executed before the promise returned from catch resolves
                node.parent.finallyBlock !== undefined && node.parent.catchClause === node)
                return true;
            // we know we can skip over the TryStatement, it always has a parent
            // if it's a function or the SourceFile, we stop. or we may potentially land in another (try/catch) block
            node = node.parent.parent;
        }
        else {
            node = node.parent;
        }
    }
    return false;
}
//# sourceMappingURL=no-return-await.js.map