"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const ymir_1 = require("@fimbul/ymir");
const tsutils_1 = require("tsutils");
const ts = require("typescript");
const utils_1 = require("../utils");
let Rule = class Rule extends ymir_1.AbstractRule {
    constructor() {
        super(...arguments);
        this.usage = undefined;
    }
    apply() {
        for (const node of this.context.getFlatAst()) {
            if (!tsutils_1.isVariableStatement(node) ||
                tsutils_1.getVariableDeclarationKind(node.declarationList) === tsutils_1.VariableDeclarationKind.Const ||
                tsutils_1.hasModifier(node.modifiers, ts.SyntaxKind.ExportKeyword, ts.SyntaxKind.DeclareKeyword) ||
                tsutils_1.isAmbientModuleBlock(node.parent))
                continue;
            for (const declaration of node.declarationList.declarations)
                if (declaration.name.kind === ts.SyntaxKind.Identifier && declaration.initializer === undefined)
                    this.checkDeclaration(declaration.name);
        }
    }
    checkDeclaration(node) {
        if (this.usage === undefined)
            this.usage = tsutils_1.collectVariableUsage(this.sourceFile);
        const variableInfo = this.usage.get(node);
        if (!variableInfo.inGlobalScope && !variableInfo.exported && !variableInfo.uses.some(utils_1.isVariableReassignment))
            this.addFindingAtNode(node, `Variable '${node.text}' is never assigned.`);
    }
};
Rule = tslib_1.__decorate([
    ymir_1.excludeDeclarationFiles
], Rule);
exports.Rule = Rule;
//# sourceMappingURL=no-unassigned-variable.js.map