"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const ts = require("typescript");
const tsutils_1 = require("tsutils");
const ymir_1 = require("@fimbul/ymir");
const utils_1 = require("../utils");
let Rule = class Rule extends ymir_1.AbstractRule {
    apply() {
        for (const node of this.context.getFlatAst()) {
            if (node.kind === ts.SyntaxKind.LabeledStatement) {
                const { label, statement } = node;
                if (!usesLabel(statement, label.text)) {
                    const start = label.getStart(this.sourceFile);
                    this.addFinding(start, label.end, `Unused label '${label.text}'.`, ymir_1.Replacement.delete(start, statement.getStart(this.sourceFile)));
                }
            }
        }
    }
};
Rule = tslib_1.__decorate([
    ymir_1.excludeDeclarationFiles
], Rule);
exports.Rule = Rule;
function usesLabel(node, label) {
    if (tsutils_1.isBreakOrContinueStatement(node))
        return node.label !== undefined && node.label.text === label;
    if (tsutils_1.isLabeledStatement(node))
        return node.label.text !== label && usesLabel(node.statement, label);
    for (const statement of utils_1.childStatements(node))
        if (usesLabel(statement, label))
            return true;
    return false;
}
//# sourceMappingURL=no-unused-label.js.map