"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const ymir_1 = require("@fimbul/ymir");
const ts = require("typescript");
const tsutils_1 = require("tsutils");
let Rule = class Rule extends ymir_1.AbstractRule {
    apply() {
        return this.checkStatements(this.sourceFile);
    }
    checkStatements(block) {
        for (const statement of block.statements) {
            if (this.shouldCheck(statement)) {
                const declareKeyword = tsutils_1.getModifier(statement, ts.SyntaxKind.DeclareKeyword);
                if (declareKeyword !== undefined) {
                    const start = declareKeyword.end - 'declare'.length;
                    this.addFinding(start, declareKeyword.end, "Using the 'declare' keyword here is redundant as the statement has no runtime value.", ymir_1.Replacement.delete(start, tsutils_1.getNextToken(declareKeyword).getStart(this.sourceFile)));
                }
            }
            else if (statement.kind === ts.SyntaxKind.ModuleDeclaration) {
                this.checkModule(statement);
            }
        }
    }
    checkModule(node) {
        if (node.body === undefined || tsutils_1.hasModifier(node.modifiers, ts.SyntaxKind.DeclareKeyword))
            return;
        if (node.body.kind === ts.SyntaxKind.ModuleDeclaration)
            return this.checkModule(node.body);
        return this.checkStatements(node.body);
    }
    shouldCheck(node) {
        switch (node.kind) {
            case ts.SyntaxKind.InterfaceDeclaration:
            case ts.SyntaxKind.TypeAliasDeclaration:
                return true;
            case ts.SyntaxKind.EnumDeclaration:
                // allow 'declare const enum' in declaration files, because it's required in declaration files
                return !this.sourceFile.isDeclarationFile && tsutils_1.hasModifier(node.modifiers, ts.SyntaxKind.ConstKeyword);
            default:
                return this.sourceFile.isDeclarationFile && tsutils_1.hasModifier(node.modifiers, ts.SyntaxKind.ExportKeyword);
        }
    }
};
Rule = tslib_1.__decorate([
    ymir_1.typescriptOnly
], Rule);
exports.Rule = Rule;
//# sourceMappingURL=no-useless-declare.js.map