"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const ymir_1 = require("@fimbul/ymir");
const ts = require("typescript");
const tsutils_1 = require("tsutils");
let Rule = class Rule extends ymir_1.AbstractRule {
    apply() {
        const { text } = this.sourceFile;
        const re = /\b(break|continue)(?:\s|\/*)/gm;
        let wrappedAst;
        for (let match = re.exec(text); match !== null; match = re.exec(text)) {
            const { node } = tsutils_1.getWrappedNodeAtPosition(wrappedAst || (wrappedAst = this.context.getWrappedAst()), match.index);
            if (tsutils_1.isBreakOrContinueStatement(node) &&
                node.label !== undefined &&
                node.label.pos - match[1].length === match.index &&
                !isLabelNecessary(node.label))
                this.addFindingAtNode(node.label, `Jump label '${node.label.text}' is unnecessary.`, ymir_1.Replacement.delete(node.label.pos, node.label.end));
        }
    }
};
Rule = tslib_1.__decorate([
    ymir_1.excludeDeclarationFiles
], Rule);
exports.Rule = Rule;
function isLabelNecessary(label) {
    let parent = label.parent;
    const isBreak = parent.kind === ts.SyntaxKind.BreakStatement;
    do {
        parent = parent.parent;
        if (tsutils_1.isIterationStatement(parent) || isBreak && parent.kind === ts.SyntaxKind.SwitchStatement) {
            // we found the closest jump target. now we need to check if it has the same label as the jump statement
            parent = parent.parent;
            while (tsutils_1.isLabeledStatement(parent)) {
                if (parent.label.text === label.text)
                    return false; // label is present on the closest jump target
                parent = parent.parent;
            }
            return true; // label is not present on the closest jump target
        }
        if (isBreak && tsutils_1.isLabeledStatement(parent) && parent.label.text === label.text)
            return true; // label is not on an IterationStatement or SwitchStatement -> breaking out of blocks always requires label
    } while (!tsutils_1.isFunctionScopeBoundary(parent) && parent.kind !== ts.SyntaxKind.SourceFile);
    return true; // label is not in scope, should never get here in correct code
}
//# sourceMappingURL=no-useless-jump-label.js.map