"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const ymir_1 = require("@fimbul/ymir");
const tsutils_1 = require("tsutils");
const ts = require("typescript");
const utils_1 = require("../utils");
let Rule = class Rule extends ymir_1.AbstractRule {
    constructor() {
        super(...arguments);
        this.strictFile = this.context.compilerOptions !== undefined && tsutils_1.isStrictCompilerOptionEnabled(this.context.compilerOptions, 'alwaysStrict')
            ? "due to the compilerOption 'alwaysStrict' this code is in strict mode" /* Option */
            : ts.isExternalModule(this.sourceFile)
                ? "ES6 modules are always in strict mode" /* Module */
                : undefined;
    }
    apply() {
        const re = /(['"])use strict\1/g;
        let wrappedAst;
        for (let match = re.exec(this.sourceFile.text); match !== null; match = re.exec(this.sourceFile.text)) {
            const { node } = tsutils_1.getWrappedNodeAtPosition(wrappedAst || (wrappedAst = this.context.getWrappedAst()), match.index);
            if (node.end === re.lastIndex &&
                tsutils_1.isStringLiteral(node) &&
                node.text === 'use strict' &&
                node.parent.kind === ts.SyntaxKind.ExpressionStatement)
                this.checkUseStrictDirective(node.parent);
        }
    }
    checkUseStrictDirective(directive) {
        const parent = directive.parent;
        if (!utils_1.hasDirectivePrologue(parent))
            return; // not a directive
        let reason = this.strictFile;
        for (const statement of parent.statements) {
            if (statement === directive) {
                if (reason === undefined && parent.kind !== ts.SyntaxKind.SourceFile)
                    reason = this.isInStrictContext(parent.parent.parent) ||
                        (this.hasUseStrictDirective(this.sourceFile) ? "a parent node is already in strict mode" /* Parent */ : undefined);
                if (reason !== undefined)
                    this.addFindingAtNode(directive, `Redundant 'use strict': ${reason}.`, ymir_1.Replacement.delete(directive.pos, directive.end));
                return;
            }
            if (!tsutils_1.isExpressionStatement(statement) || !tsutils_1.isStringLiteral(statement.expression))
                return; // not a directive
            if (reason === undefined && statement.expression.getText(this.sourceFile).slice(1, -1) === 'use strict')
                reason = "there is already a 'use strict' directive in this prologue" /* Antedecent */;
        }
    }
    isInStrictContext(node) {
        while (true) {
            switch (node.kind) {
                case ts.SyntaxKind.ClassDeclaration:
                case ts.SyntaxKind.ClassExpression:
                    return "classes are always in strict mode" /* Class */;
                case ts.SyntaxKind.SourceFile:
                    return; // SourceFile was already checked
            }
            if (utils_1.hasDirectivePrologue(node)) {
                if (this.hasUseStrictDirective(node))
                    return "a parent node is already in strict mode" /* Parent */;
                node = node.parent.parent;
            }
            else {
                node = node.parent;
            }
        }
    }
    hasUseStrictDirective(node) {
        for (const statement of node.statements) {
            if (!tsutils_1.isExpressionStatement(statement) || !tsutils_1.isStringLiteral(statement.expression))
                break;
            if (statement.expression.getText(this.sourceFile).slice(1, -1) === 'use strict')
                return true;
        }
        return false;
    }
};
Rule = tslib_1.__decorate([
    ymir_1.excludeDeclarationFiles
], Rule);
exports.Rule = Rule;
//# sourceMappingURL=no-useless-strict.js.map