"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const ymir_1 = require("@fimbul/ymir");
const ts = require("typescript");
const tsutils_1 = require("tsutils");
let Rule = class Rule extends ymir_1.TypedRule {
    apply() {
        const re = /\b(?:isNaN|isFinite)\s*[/(]/g;
        let wrappedAst;
        for (let match = re.exec(this.sourceFile.text); match !== null; match = re.exec(this.sourceFile.text)) {
            const { node } = tsutils_1.getWrappedNodeAtPosition(wrappedAst || (wrappedAst = this.context.getWrappedAst()), match.index);
            if (!tsutils_1.isIdentifier(node) || node.text !== 'isNaN' && node.text !== 'isFinite' || node.end - node.text.length !== match.index)
                continue;
            const parent = node.parent;
            if (tsutils_1.isCallExpression(parent) && parent.expression === node && parent.arguments.length === 1 &&
                this.isCorrectArgumentType(parent.arguments[0]))
                this.addFinding(match.index, node.end, `Prefer 'Number.${node.text}' over '${node.text}'.`, ymir_1.Replacement.append(match.index, 'Number.'));
        }
    }
    isCorrectArgumentType(arg) {
        const type = this.checker.getTypeAtLocation(arg);
        return tsutils_1.unionTypeParts(this.checker.getBaseConstraintOfType(type) || type).every((t) => (t.flags & ts.TypeFlags.NumberLike) !== 0);
    }
};
Rule = tslib_1.__decorate([
    ymir_1.excludeDeclarationFiles,
    ymir_1.requireLibraryFile('lib.es2015.core.d.ts')
], Rule);
exports.Rule = Rule;
//# sourceMappingURL=prefer-number-methods.js.map