"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const ymir_1 = require("@fimbul/ymir");
const ts = require("typescript");
const tsutils_1 = require("tsutils");
let Rule = class Rule extends ymir_1.TypedRule {
    apply() {
        for (const node of this.context.getFlatAst())
            if (tsutils_1.isExpressionStatement(node) && tsutils_1.isCallExpression(node.expression) && this.returnsNever(node.expression))
                this.addFindingAtNode(node, `This call never returns. Consider ${isReturnAllowed(node) ? 'return' : 'throw'}ing the result for better control flow analysis and type inference.`);
    }
    returnsNever(node) {
        const returnType = this.checker.getApparentType(this.checker.getResolvedSignature(node).getReturnType());
        return (returnType.flags & ts.TypeFlags.Never) !== 0;
    }
};
Rule = tslib_1.__decorate([
    ymir_1.excludeDeclarationFiles
], Rule);
exports.Rule = Rule;
function isReturnAllowed(node) {
    while (true) {
        node = node.parent;
        if (node.kind === ts.SyntaxKind.SourceFile || node.kind === ts.SyntaxKind.ModuleBlock)
            return false;
        if (tsutils_1.isFunctionScopeBoundary(node))
            return true;
    }
}
//# sourceMappingURL=return-never-call.js.map