"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const ymir_1 = require("@fimbul/ymir");
const tsutils_1 = require("tsutils");
const ts = require("typescript");
const utils_1 = require("../utils");
let Rule = class Rule extends ymir_1.TypedRule {
    constructor() {
        super(...arguments);
        this.reported = new Set();
    }
    apply() {
        for (const node of utils_1.tryStatements(this.context)) {
            if (!this.reported.has(node.pos) &&
                isInAsyncFunction(node)) {
                node.tryBlock.statements.forEach(this.visitStatement, this);
                // special handling for catchClause only if finallyBlock is present
                if (node.catchClause !== undefined && node.finallyBlock !== undefined)
                    node.catchClause.block.statements.forEach(this.visitStatement, this);
            }
        }
    }
    visitStatement(node) {
        if (tsutils_1.isReturnStatement(node)) {
            if (node.expression !== undefined)
                this.checkReturnExpression(node.expression);
            return;
        }
        if (node.kind === ts.SyntaxKind.TryStatement)
            this.reported.add(node.pos);
        for (const statement of utils_1.childStatements(node))
            this.visitStatement(statement);
    }
    checkReturnExpression(node) {
        const { pos } = node;
        while (tsutils_1.isParenthesizedExpression(node))
            node = node.expression;
        if (node.kind === ts.SyntaxKind.AwaitExpression)
            return;
        if (tsutils_1.isThenableType(this.checker, node))
            this.addFinding(pos - 'return'.length, pos, "Missing 'await' of Promise returned inside try-catch.", needsParens(node)
                ? [
                    ymir_1.Replacement.append(pos, ' await'),
                    ymir_1.Replacement.append(node.getStart(this.sourceFile), '('),
                    ymir_1.Replacement.append(node.end, ')'),
                ]
                : ymir_1.Replacement.append(pos, ' await'));
    }
};
Rule = tslib_1.__decorate([
    ymir_1.excludeDeclarationFiles
], Rule);
exports.Rule = Rule;
function isInAsyncFunction(node) {
    do {
        node = node.parent;
        if (node.kind === ts.SyntaxKind.SourceFile)
            return false;
    } while (!tsutils_1.isFunctionScopeBoundary(node));
    return utils_1.isAsyncFunction(node);
}
function needsParens(node) {
    switch (node.kind) {
        case ts.SyntaxKind.ConditionalExpression:
        case ts.SyntaxKind.BinaryExpression:
            return node.parent.kind !== ts.SyntaxKind.ParenthesizedExpression;
        default:
            return false;
    }
}
//# sourceMappingURL=try-catch-return-await.js.map